@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: admin.c,v 1.17 2003/05/23 10:41:26 adam Exp $
+ * $Id: admin.c,v 1.19 2005/01/16 21:51:49 adam Exp $
  */
 
 #include <stdio.h>
@@ -10,9 +10,13 @@
 #include <time.h>
 #include <assert.h>
 
-#if HAVE_FNMATCH_H
+#if HAVE_DIRENT_H
 #include <dirent.h>
+#endif
+#if HAVE_FNMATCH_H
 #include <fnmatch.h>
+#endif
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 
