@@ -1,15 +1,22 @@
 /* 
- * Copyright (c) 2003, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: bertorture.c,v 1.1 2003/03/11 11:06:35 adam Exp $
+ * $Id: bertorture.c,v 1.3 2005/01/16 21:51:49 adam Exp $
  */
 
 #include <signal.h>
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
 #include <fcntl.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 
