@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2004, Index Data.
+ * Copyright (C) 1995-2005, Index Data ApS
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation, in whole or in part, for any purpose, is hereby granted,
@@ -23,7 +23,7 @@
  * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  *
- * $Id: nmem.h,v 1.13 2004/10/15 00:18:59 adam Exp $
+ * $Id: nmem.h,v 1.16 2005/06/03 20:30:30 adam Exp $
  */
 
 /**
@@ -35,6 +35,8 @@
  */
 #ifndef NMEM_H
 #define NMEM_H
+
+#include <stddef.h>
 #include <yaz/yconfig.h>
 
 #define NMEM_DEBUG 0
@@ -45,20 +47,9 @@
 
 YAZ_BEGIN_CDECL
 
-typedef struct nmem_block
-{
-    char *buf;              /* memory allocated in this block */
-    int size;               /* size of buf */
-    int top;                /* top of buffer */
-    struct nmem_block *next;
-} nmem_block;
+typedef struct nmem_block nmem_block;
 
-typedef struct nmem_control
-{
-    int total;
-    nmem_block *blocks;
-    struct nmem_control *next;
-} nmem_control;
+typedef struct nmem_control nmem_control;
 
 typedef struct nmem_mutex *NMEM_MUTEX;
 YAZ_EXPORT void nmem_mutex_create(NMEM_MUTEX *);
@@ -72,6 +63,12 @@ YAZ_EXPORT void nmem_reset(NMEM n);
 YAZ_EXPORT int nmem_total(NMEM n);
 YAZ_EXPORT char *nmem_strdup (NMEM mem, const char *src);
 YAZ_EXPORT char *nmem_strdupn (NMEM mem, const char *src, size_t n);
+YAZ_EXPORT void nmem_strsplit_blank(NMEM nmem, const char *dstr,
+				    char ***darray, int *num);
+YAZ_EXPORT void nmem_strsplit(NMEM nmem, const char *delim,
+			      const char *dstr,
+			      char ***darray, int *num);
+
 YAZ_EXPORT int *nmem_intdup (NMEM mem, int v);
 YAZ_EXPORT void nmem_transfer (NMEM dst, NMEM src);
 
