@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2004, Index Data.
+ * Copyright (C) 1995-2005, Index Data ApS
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation, in whole or in part, for any purpose, is hereby granted,
@@ -23,7 +23,7 @@
  * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  *
- * $Id: xmalloc.h,v 1.4 2004/10/15 00:18:59 adam Exp $
+ * $Id: xmalloc.h,v 1.6 2005/01/16 21:51:49 adam Exp $
  */
 /**
  * \file xmalloc.h
@@ -33,8 +33,7 @@
 #ifndef XMALLOC_H
 #define XMALLOC_H
 
-#include <sys/types.h>
-#include <stdlib.h>
+#include <stddef.h>
 
 #include <yaz/yconfig.h>
 
