@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: eventl.c,v 1.4 2004/12/13 14:21:55 heikki Exp $
+ * $Id: eventl.c,v 1.8 2005/03/01 20:37:01 adam Exp $
  */
 
 /**
@@ -13,16 +13,27 @@
  * Server. It uses select(2).
  */
 
-#include <stdio.h>
 #include <assert.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef WIN32
 #include <winsock.h>
-#else
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdlib.h>
-#include <errno.h>
-#include <string.h>
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 
 #include <yaz/yconfig.h>
 #include <yaz/log.h>
@@ -44,7 +55,7 @@
 static int log_level=0;
 static int log_level_initialized=0;
 
-IOCHAN iochan_create(int fd, IOC_CALLBACK cb, int flags)
+IOCHAN iochan_create(int fd, IOC_CALLBACK cb, int flags, int chan_id)
 {
     IOCHAN new_iochan;
 
@@ -63,6 +74,7 @@ IOCHAN iochan_create(int fd, IOC_CALLBACK cb, int flags)
     new_iochan->force_event = 0;
     new_iochan->last_event = new_iochan->max_idle = 0;
     new_iochan->next = NULL;
+    new_iochan->chan_id = chan_id;
     return new_iochan;
 }
 
