@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: eventl.h,v 1.2 2004/10/15 00:19:00 adam Exp $
+ * $Id: eventl.h,v 1.5 2005/03/01 20:37:01 adam Exp $
  */
 
 /**
@@ -39,6 +39,7 @@ int force_event;
     time_t max_idle;
     
     struct iochan *next;
+    int chan_id; /* listening port (0 if none ) */
 } *IOCHAN;
 
 #define iochan_destroy(i) (void)((i)->destroyed = 1)
@@ -57,7 +58,7 @@ int force_event;
 #define iochan_getnext(i) ((i)->next)
 #define iochan_settimeout(i, t) ((i)->max_idle = (t), (i)->last_event = time(0))
 
-IOCHAN iochan_create(int fd, IOC_CALLBACK cb, int flags);
+IOCHAN iochan_create(int fd, IOC_CALLBACK cb, int flags, int port);
 int event_loop(IOCHAN *iochans);
 void statserv_remove (IOCHAN pIOChannel);
 #endif
