@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: log.c,v 1.20 2005/01/07 10:28:28 heikki Exp $
+ * $Id: log.c,v 1.24 2005/05/18 12:30:00 adam Exp $
  */
 
 /**
@@ -83,10 +83,10 @@ static struct {
     { YLOG_LOG,    "log"  },
     { YLOG_ERRNO,  ""},
     { YLOG_MALLOC, "malloc"},
- /*   { YLOG_APP,    "app"  }, */
+    { YLOG_APP,     "app"  },
     { YLOG_NOTIME, "notime" },
- /*   { YLOG_APP2,   "app2" }, */
- /*   { YLOG_APP3,   "app3" }, */
+    { YLOG_APP2,   "app2" }, 
+    { YLOG_APP3,   "app3" },
     { YLOG_ALL,    "all"  },
     { YLOG_FLUSH,  "flush" },
     { YLOG_LOGLVL, "loglevel" }, 
@@ -174,6 +174,8 @@ static void rotate_log()
 
 void yaz_log_init_level (int level)
 {
+    if (!mutex_init_flag)
+        init_mutex();
     if ( (l_level & YLOG_FLUSH) != (level & YLOG_FLUSH) )
     {
         l_level = level;
@@ -361,6 +363,8 @@ static char *clean_name(const char *name, int len, char *namebuf, int buflen)
 static int define_module_bit(const char *name)
 {
     int i;
+    if (!mutex_init_flag)
+        init_mutex();
     for (i = 0; mask_names[i].name; i++)
         ;
     if ( (i>=MAX_MASK_NAMES) || (next_log_bit >= 1<<31 ))
@@ -381,6 +385,8 @@ int yaz_log_module_level(const char *name)
     int i;
     char clean[255];
     char *n = clean_name(name, strlen(name), clean, sizeof(clean));
+    if (!mutex_init_flag)
+        init_mutex();
     for (i = 0; mask_names[i].name; i++)
         if (0==strcmp(n,mask_names[i].name))
         {
