@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: marcdisp.c,v 1.20 2005/03/07 06:07:33 adam Exp $
+ * $Id: marcdisp.c,v 1.22 2005/06/07 19:25:38 adam Exp $
  */
 
 /**
@@ -214,8 +214,7 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
     {
 	char str[40];
 
-        if (mt->xml)
-            wrbuf_puts (wr, "<!--\n");
+	wrbuf_puts (wr, "<!--\n");
 	sprintf (str, "Record length         %5d\n", record_length);
 	wrbuf_puts (wr, str);
 	sprintf (str, "Indicator length      %5d\n", indicator_length);
@@ -230,8 +229,7 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
 	wrbuf_puts (wr, str);
 	sprintf (str, "Length implementation %5d\n", length_implementation);
 	wrbuf_puts (wr, str);
-        if (mt->xml)
-            wrbuf_puts (wr, "-->\n");
+	wrbuf_puts (wr, "-->\n");
     }
 
     /* first pass. determine length of directory & base of data */
@@ -380,8 +378,6 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
         switch(mt->xml)
         {
         case YAZ_MARC_LINE:
-            if (mt->debug)
-                wrbuf_puts (wr, "Tag: ");
             wrbuf_puts (wr, tag);
             wrbuf_puts (wr, " ");
             break;
@@ -419,8 +415,6 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
 		    wrbuf_putc(wr, buf[i]);
 		    break;
                 case YAZ_MARC_LINE:
-                    if (mt->debug)
-                        wrbuf_puts (wr, " Ind: ");
                     wrbuf_putc(wr, buf[i]);
                     break;
                 case YAZ_MARC_SIMPLEXML:
@@ -448,11 +442,6 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
             if (identifier_flag)
                 wrbuf_puts (wr, "\n");
         }
-        if (mt->xml == YAZ_MARC_LINE)
-        {
-            if (mt->debug)
-                wrbuf_puts (wr, " Fields: ");
-        }
         if (identifier_flag)
         {
             while (i < end_offset &&
@@ -572,7 +561,10 @@ int yaz_marc_decode_wrbuf (yaz_marc_t mt, const char *buf, int bsize, WRBUF wr)
 int yaz_marc_decode_buf (yaz_marc_t mt, const char *buf, int bsize,
                          char **result, int *rsize)
 {
-    int r = yaz_marc_decode_wrbuf(mt, buf, bsize, mt->m_wr);
+    int r;
+
+    wrbuf_rewind(mt->m_wr);
+    r = yaz_marc_decode_wrbuf(mt, buf, bsize, mt->m_wr);
     if (result)
         *result = wrbuf_buf(mt->m_wr);
     if (rsize)
