@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2005, Index Data.
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: nmem.c,v 1.11 2005/01/05 10:23:42 adam Exp $
+ * $Id: nmem.c,v 1.18 2005/06/07 19:47:31 adam Exp $
  */
 
 /**
@@ -20,6 +20,7 @@
 #endif
 
 #include <assert.h>
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <stddef.h>
@@ -42,6 +43,21 @@
 
 #define NMEM_CHUNK (4*1024)
 
+struct nmem_block
+{
+    char *buf;              /* memory allocated in this block */
+    size_t size;            /* size of buf */
+    size_t top;             /* top of buffer */
+    struct nmem_block *next;
+};
+
+struct nmem_control
+{
+    int total;
+    nmem_block *blocks;
+    struct nmem_control *next;
+};
+
 struct align {
     char x;
     union {
@@ -59,8 +75,8 @@ struct align {
 
 #define NMEM_ALIGN (offsetof(struct align, u))
 
-static int log_level=0;
-static int log_level_initialized=0;
+static int log_level = 0;
+static int log_level_initialized = 0;
 
 #ifdef WIN32
 static CRITICAL_SECTION critical_section;
@@ -93,13 +109,6 @@ struct nmem_mutex {
 
 YAZ_EXPORT void nmem_mutex_create(NMEM_MUTEX *p)
 {
-    if (!log_level_initialized)
-    {
-        log_level=yaz_log_module_level("nmem");
-        log_level_initialized=1;
-    }
-
-    NMEM_ENTER;
     if (!*p)
     {
 	*p = (NMEM_MUTEX) malloc (sizeof(**p));
@@ -111,7 +120,12 @@ YAZ_EXPORT void nmem_mutex_create(NMEM_MUTEX *p)
         pth_mutex_init (&(*p)->m_handle);
 #endif
     }
-    NMEM_LEAVE;
+    if (!log_level_initialized)
+    {
+        log_level_initialized = 1;
+        log_level = yaz_log_module_level("nmem");
+    }
+
 }
 
 YAZ_EXPORT void nmem_mutex_enter(NMEM_MUTEX p)
@@ -140,7 +154,6 @@ YAZ_EXPORT void nmem_mutex_leave(NMEM_MUTEX p)
 
 YAZ_EXPORT void nmem_mutex_destroy(NMEM_MUTEX *p)
 {
-    NMEM_ENTER;
     if (*p)
     {
 #ifdef WIN32
@@ -149,7 +162,6 @@ YAZ_EXPORT void nmem_mutex_destroy(NMEM_MUTEX *p)
 	free (*p);
 	*p = 0;
     }
-    NMEM_LEAVE;
 }
 
 static nmem_block *freelist = NULL;        /* "global" freelists */
@@ -199,12 +211,12 @@ void nmem_print_list_l (int level)
 /*
  * acquire a block with a minimum of size free bytes.
  */
-static nmem_block *get_block(int size)
+static nmem_block *get_block(size_t size)
 {
     nmem_block *r, *l;
 
     if (log_level)
-        yaz_log (log_level, "nmem get_block size=%d", size);
+        yaz_log (log_level, "nmem get_block size=%ld", (long) size);
 
     for (r = freelist, l = 0; r; l = r, r = r->next)
     	if (r->size >= size)
@@ -276,7 +288,7 @@ void *nmem_malloc(NMEM n, int size)
 #endif
     NMEM_ENTER;
     p = n->blocks;
-    if (!p || p->size - p->top < size)
+    if (!p || p->size < size + p->top)
     {
     	p = get_block(size);
 	p->next = n->blocks;
@@ -307,8 +319,8 @@ NMEM nmem_create(void)
 #endif
     if (!log_level_initialized)
     {
-        log_level=yaz_log_module_level("nmem");
-        log_level_initialized=1;
+        log_level = yaz_log_module_level("nmem");
+        log_level_initialized = 1;
     }
     
     NMEM_ENTER;
@@ -416,24 +428,23 @@ void nmem_critical_leave (void)
 
 void nmem_init (void)
 {
-    if (!log_level_initialized)
-    {
-        log_level=yaz_log_module_level("nmem");
-        log_level_initialized=1;
-    }
     
     if (++nmem_init_flag == 1)
     {
 #ifdef WIN32
 	InitializeCriticalSection(&critical_section);
 #elif YAZ_GNU_THREADS
-	yaz_log (log_level, "pth_init"); /* ??? */
         pth_init ();
 #endif
 	nmem_active_no = 0;
 	freelist = NULL;
 	cfreelist = NULL;
     }
+    if (!log_level_initialized)
+    {
+        log_level = yaz_log_module_level("nmem");
+        log_level_initialized = 1;
+    }
 }
 
 void nmem_exit (void)
