@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 1995-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
  * NT threaded server code by
  *   Chas Woodfield, Fretwell Downing Informatics.
  *
- * $Id: statserv.c,v 1.17 2005/01/03 11:56:08 adam Exp $
+ * $Id: statserv.c,v 1.30 2005/05/08 07:13:57 adam Exp $
  */
 
 /**
@@ -15,16 +15,32 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #ifdef WIN32
 #include <process.h>
 #include <winsock.h>
 #include <direct.h>
 #include "service.h"
-#else
+#endif
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#if HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#if HAVE_PWD_H
 #include <pwd.h>
 #endif
 
+#if HAVE_XML2
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+#include <libxml/xinclude.h>
+#endif
+
 #if YAZ_POSIX_THREADS
 #include <pthread.h>
 #elif YAZ_GNU_THREADS
@@ -48,8 +64,23 @@
 
 static IOCHAN pListener = NULL;
 
+static char gfs_root_dir[FILENAME_MAX+1];
+static struct gfs_server *gfs_server_list = 0;
+static struct gfs_listen *gfs_listen_list = 0;
+static NMEM gfs_nmem = 0;
+
 static char *me = "statserver"; /* log prefix */
 static char *programname="statserver"; /* full program name */
+#ifdef WIN32
+DWORD current_control_tls;
+static int init_control_tls = 0;
+#elif YAZ_POSIX_THREADS
+static pthread_key_t current_control_tls;
+static int init_control_tls = 0;
+#else
+static statserv_options_block *current_control_block = 0;
+#endif
+
 /*
  * default behavior.
  */
@@ -88,27 +119,427 @@ statserv_options_block control_block = {
     0,                          /* SOAP handlers */
     "",                         /* PID fname */
     0,                          /* background daemon */
-    ""                          /* SSL certificate filename */
+    "",                         /* SSL certificate filename */
+    ""                          /* XML config filename */
 };
 
 static int max_sessions = 0;
 
-static int logbits_set=0;
-static int log_session=0;
-static int log_server=0;
+static int logbits_set = 0;
+static int log_session = 0;
+static int log_server = 0;
 
 /** get_logbits sets global loglevel bits */
 static void get_logbits(int force)
 { /* needs to be called after parsing cmd-line args that can set loglevels!*/
     if (force || !logbits_set)
     {
-        logbits_set=1;
-        log_session=yaz_log_module_level("session");
-        log_server=yaz_log_module_level("server");
+        logbits_set = 1;
+        log_session = yaz_log_module_level("session");
+        log_server = yaz_log_module_level("server");
     }
 }
 
 
+static int add_listener(char *where, int listen_id);
+
+#if HAVE_XML2
+static xmlDocPtr xml_config_doc = 0;
+#endif
+
+#if HAVE_XML2
+static xmlNodePtr xml_config_get_root()
+{
+    xmlNodePtr ptr = 0;
+    if (xml_config_doc)
+    {
+	ptr = xmlDocGetRootElement(xml_config_doc);
+	if (!ptr || ptr->type != XML_ELEMENT_NODE ||
+	    strcmp((const char *) ptr->name, "yazgfs"))
+	{
+	    yaz_log(YLOG_WARN, "Bad/missing root element for config %s",
+		    control_block.xml_config);
+	    return 0;
+	
+	}
+    }
+    return ptr;
+}
+#endif
+
+#if HAVE_XML2
+static char *nmem_dup_xml_content(NMEM n, xmlNodePtr ptr)
+{
+    unsigned char *cp;
+    xmlNodePtr p;
+    int len = 1;  /* start with 1, because of trailing 0 */
+    char *str;
+    int first = 1; /* whitespace lead flag .. */
+    /* determine length */
+    for (p = ptr; p; p = p->next)
+    {
+	if (p->type == XML_TEXT_NODE)
+	    len += strlen(p->content);
+    }
+    /* now allocate for the string */
+    str = nmem_malloc(n, len);
+    *str = '\0'; /* so we can use strcat */
+    for (p = ptr; p; p = p->next)
+    {
+	if (p->type == XML_TEXT_NODE)
+	{
+	    cp = p->content;
+	    if (first)
+	    {
+		while(*cp && isspace(*cp))
+		    cp++;
+		if (*cp)
+		    first = 0;  /* reset if we got non-whitespace out */
+	    }
+	    strcat(str, cp); /* append */
+	}
+    }
+    /* remove trailing whitespace */
+    cp = strlen(str) + str;
+    while ((char*) cp != str && isspace(cp[-1]))
+	cp--;
+    *cp = '\0';
+    /* return resulting string */
+    return str;
+}
+#endif
+
+static struct gfs_server * gfs_server_new()
+{
+    struct gfs_server *n = nmem_malloc(gfs_nmem, sizeof(*n));
+    memcpy(&n->cb, &control_block, sizeof(control_block));
+    n->next = 0;
+    n->host = 0;
+    n->listen_ref = 0;
+    n->cql_transform = 0;
+    n->server_node_ptr = 0;
+    n->directory = 0;
+    return n;
+}
+
+static struct gfs_listen * gfs_listen_new(const char *id, 
+					  const char *address)
+{
+    struct gfs_listen *n = nmem_malloc(gfs_nmem, sizeof(*n));
+    if (id)
+	n->id = nmem_strdup(gfs_nmem, id);
+    else
+	n->id = 0;
+    n->next = 0;
+    n->address = nmem_strdup(gfs_nmem, address);
+    return n;
+}
+
+static void gfs_server_chdir(struct gfs_server *gfs)
+{
+    if (gfs_root_dir[0])
+    {
+	if (chdir(gfs_root_dir))
+	    yaz_log(YLOG_WARN|YLOG_ERRNO, "chdir %s", gfs_root_dir);
+    }
+    if (gfs->directory)
+    {
+	if (chdir(gfs->directory))
+	    yaz_log(YLOG_WARN|YLOG_ERRNO, "chdir %s",
+		    gfs->directory);
+    }
+}
+
+int control_association(association *assoc, const char *host, int force_open)
+{
+    char vhost[128], *cp;
+    if (host)
+    {
+	strncpy(vhost, host, 127);
+	vhost[127] = '\0';
+	cp = strchr(vhost, ':');
+	if (cp)
+	    *cp = '\0';
+	host = vhost;
+    }
+    if (control_block.xml_config[0])
+    {
+	struct gfs_server *gfs;
+	for (gfs = gfs_server_list; gfs; gfs = gfs->next)
+	{
+	    int listen_match = 0;
+	    int host_match = 0;
+	    if ( !gfs->host || (host && gfs->host && !strcmp(host, gfs->host)))
+		host_match = 1;
+	    if (!gfs->listen_ref ||
+		gfs->listen_ref == assoc->client_chan->chan_id)
+		listen_match = 1;
+	    if (listen_match && host_match)
+	    {
+		if (force_open ||
+		    (assoc->last_control != &gfs->cb && assoc->backend))
+		{
+		    statserv_setcontrol(assoc->last_control);
+		    if (assoc->backend && assoc->init)
+		    {
+			gfs_server_chdir(gfs);
+			(assoc->last_control->bend_close)(assoc->backend);
+		    }
+		    assoc->backend = 0;
+		    xfree(assoc->init);
+		    assoc->init = 0;
+		}
+		assoc->cql_transform = gfs->cql_transform;
+		assoc->server_node_ptr = gfs->server_node_ptr;
+		assoc->last_control = &gfs->cb;
+		statserv_setcontrol(&gfs->cb);
+		gfs_server_chdir(gfs);
+		yaz_log(YLOG_DEBUG, "server select: %s", gfs->cb.configname);
+		return 1;
+	    }
+	}
+	statserv_setcontrol(0);
+	assoc->last_control = 0;
+	assoc->cql_transform = 0;
+	assoc->server_node_ptr = 0;
+	yaz_log(YLOG_DEBUG, "server select: no match");
+	return 0;
+    }
+    else
+    {
+	statserv_setcontrol(&control_block);
+	assoc->last_control = &control_block;
+	assoc->cql_transform = 0;
+	assoc->server_node_ptr = 0;
+	yaz_log(YLOG_DEBUG, "server select: config=%s", control_block.configname);
+	return 1;
+    }
+}
+
+static void xml_config_read()
+{
+    struct gfs_server **gfsp = &gfs_server_list;
+    struct gfs_listen **gfslp = &gfs_listen_list;
+#if HAVE_XML2
+    xmlNodePtr ptr = xml_config_get_root();
+
+    if (!ptr)
+	return;
+    for (ptr = ptr->children; ptr; ptr = ptr->next)
+    {
+	struct _xmlAttr *attr;
+	if (ptr->type != XML_ELEMENT_NODE)
+	    continue;
+	attr = ptr->properties;
+	if (!strcmp((const char *) ptr->name, "listen"))
+	{
+	    /*
+	      <listen id="listenerid">tcp:@:9999</listen>
+	    */
+	    const char *id = 0;
+	    const char *address =
+		nmem_dup_xml_content(gfs_nmem, ptr->children);
+	    for ( ; attr; attr = attr->next)
+		if (!strcmp(attr->name, "id")
+		    && attr->children && attr->children->type == XML_TEXT_NODE)
+		    id = nmem_dup_xml_content(gfs_nmem, attr->children);
+	    if (address)
+	    {
+		*gfslp = gfs_listen_new(id, address);
+		gfslp = &(*gfslp)->next;
+		*gfslp = 0; /* make listener list consistent for search */
+	    }
+	}
+	else if (!strcmp((const char *) ptr->name, "server"))
+	{
+	    xmlNodePtr ptr_server = ptr;
+	    xmlNodePtr ptr;
+	    const char *listenref = 0;
+	    const char *id = 0;
+
+	    for ( ; attr; attr = attr->next)
+		if (!strcmp(attr->name, "listenref") && attr->children &&
+		    attr->children->type == XML_TEXT_NODE)
+		    listenref = nmem_dup_xml_content(gfs_nmem, attr->children);
+		else if (!strcmp(attr->name, "id") && attr->children &&
+			 attr->children->type == XML_TEXT_NODE)
+		    id = nmem_dup_xml_content(gfs_nmem, attr->children);
+		else
+		    yaz_log(YLOG_WARN, "Unknown attribute '%s' for server",
+			    attr->name);
+	    *gfsp = gfs_server_new();
+	    (*gfsp)->server_node_ptr = ptr_server;
+	    if (listenref)
+	    {
+		int id_no;
+		struct gfs_listen *gl = gfs_listen_list;
+		for (id_no = 1; gl; gl = gl->next, id_no++)
+		    if (gl->id && !strcmp(gl->id, listenref))
+		    {
+			(*gfsp)->listen_ref = id_no;
+			break;
+		    }
+		if (!gl)
+		    yaz_log(YLOG_WARN, "Non-existent listenref '%s' in server "
+			    "config element", listenref);
+	    }
+	    for (ptr = ptr_server->children; ptr; ptr = ptr->next)
+	    {
+		if (ptr->type != XML_ELEMENT_NODE)
+		    continue;
+		if (!strcmp((const char *) ptr->name, "host"))
+		{
+		    (*gfsp)->host = nmem_dup_xml_content(gfs_nmem,
+							 ptr->children);
+		}
+		else if (!strcmp((const char *) ptr->name, "config"))
+		{
+		    strcpy((*gfsp)->cb.configname,
+			   nmem_dup_xml_content(gfs_nmem, ptr->children));
+		}
+		else if (!strcmp((const char *) ptr->name, "cql2rpn"))
+		{
+		    (*gfsp)->cql_transform = cql_transform_open_fname(
+			nmem_dup_xml_content(gfs_nmem, ptr->children)
+			);
+		}
+		else if (!strcmp((const char *) ptr->name, "directory"))
+		{
+		    (*gfsp)->directory = 
+			nmem_dup_xml_content(gfs_nmem, ptr->children);
+		}
+	    }
+	    gfsp = &(*gfsp)->next;
+	}
+    }
+#endif
+    *gfsp = 0;
+}
+
+static void xml_config_open()
+{
+    if (!getcwd(gfs_root_dir, FILENAME_MAX))
+    {
+	yaz_log(YLOG_WARN|YLOG_ERRNO, "getcwd failed");
+	gfs_root_dir[0] = '\0';
+    }
+#ifdef WIN32
+    init_control_tls = 1;
+    current_control_tls = TlsAlloc();
+#elif YAZ_POSIX_THREADS
+    init_control_tls = 1;
+    pthread_key_create(&current_control_tls, 0);
+#endif
+    
+    gfs_nmem = nmem_create();
+#if HAVE_XML2
+    if (control_block.xml_config[0] == '\0')
+	return;
+
+    if (!xml_config_doc)
+    {
+	xml_config_doc = xmlParseFile(control_block.xml_config);
+	if (!xml_config_doc)
+	{
+	    yaz_log(YLOG_FATAL, "Could not parse %s", control_block.xml_config);
+	    exit(1);
+	}
+	else
+	{
+	    int noSubstitutions = xmlXIncludeProcess(xml_config_doc);
+	    if (noSubstitutions == -1)
+	    {
+		yaz_log(YLOG_WARN, "XInclude processing failed for config %s",
+			control_block.xml_config);
+		exit(1);
+	    }
+	}
+    }
+    xml_config_read();
+#endif
+}
+
+static void xml_config_close()
+{
+#if HAVE_XML2
+    if (xml_config_doc)
+    {
+	xmlFreeDoc(xml_config_doc);
+	xml_config_doc = 0;
+    }
+#endif
+    gfs_server_list = 0;
+    nmem_destroy(gfs_nmem);
+#ifdef WIN32
+    if (init_control_tls)
+	TlsFree(current_control_tls);
+#elif YAZ_POSIX_THREADS
+    if (init_control_tls)
+        pthread_key_delete(current_control_tls);
+#endif
+}
+
+static void xml_config_add_listeners()
+{
+    struct gfs_listen *gfs = gfs_listen_list;
+    int id_no;
+
+    for (id_no = 1; gfs; gfs = gfs->next, id_no++)
+    {
+	if (gfs->address)
+	    add_listener(gfs->address, id_no);
+    }
+}
+
+static void xml_config_bend_start()
+{
+    if (control_block.xml_config[0])
+    {
+	struct gfs_server *gfs = gfs_server_list;
+	for (; gfs; gfs = gfs->next)
+	{
+	    yaz_log(YLOG_DEBUG, "xml_config_bend_start config=%s",
+		    gfs->cb.configname);
+	    statserv_setcontrol(&gfs->cb);
+	    if (control_block.bend_start)
+	    {
+		gfs_server_chdir(gfs);
+		(control_block.bend_start)(&gfs->cb);
+	    }
+	}
+    }
+    else
+    {
+	yaz_log(YLOG_DEBUG, "xml_config_bend_start default config");
+	statserv_setcontrol(&control_block);
+	if (control_block.bend_start)
+	    (*control_block.bend_start)(&control_block);
+    }
+}
+
+static void xml_config_bend_stop()
+{
+    if (control_block.xml_config[0])
+    {
+	struct gfs_server *gfs = gfs_server_list;
+	for (; gfs; gfs = gfs->next)
+	{
+	    yaz_log(YLOG_DEBUG, "xml_config_bend_stop config=%s",
+		    gfs->cb.configname);
+	    statserv_setcontrol(&gfs->cb);
+	    if (control_block.bend_stop)
+		(control_block.bend_stop)(&gfs->cb);
+	}
+    }
+    else
+    {
+	yaz_log(YLOG_DEBUG, "xml_config_bend_stop default config");
+	statserv_setcontrol(&control_block);
+	if (control_block.bend_stop)
+	    (*control_block.bend_stop)(&control_block);
+    }
+}
+
 /*
  * handle incoming connect requests.
  * The dynamic mode is a bit tricky mostly because we want to avoid
@@ -170,7 +601,7 @@ void statserv_remove(IOCHAN pIOChannel)
         ThreadList *pNextThread;
         ThreadList *pPrevThread =NULL;
 
-        /* Step through alll the threads */
+        /* Step through all the threads */
         for (; pCurrentThread != NULL; pCurrentThread = pNextThread)
         {
             /* We only need to compare on the IO Channel */
@@ -258,7 +689,7 @@ void statserv_closedown()
         /* Now we can really do something */
         if (iHandles > 0)
         {
-            logf (log_server, "waiting for %d to die", iHandles);
+            yaz_log(log_server, "waiting for %d to die", iHandles);
             /* This will now wait, until all the threads close */
             WaitForMultipleObjects(iHandles, pThreadHandles, TRUE, INFINITE);
 
@@ -266,11 +697,11 @@ void statserv_closedown()
             free(pThreadHandles);
         }
 
-	if (control_block.bend_stop)
-	    (*control_block.bend_stop)(&control_block);
+	xml_config_bend_stop();
         /* No longer require the critical section, since all threads are dead */
         DeleteCriticalSection(&Thread_CritSect);
     }
+    xml_config_close();
 }
 
 void __cdecl event_loop_thread (IOCHAN iochan)
@@ -282,6 +713,7 @@ void __cdecl event_loop_thread (IOCHAN iochan)
 static void listener(IOCHAN h, int event)   
 {
     COMSTACK line = (COMSTACK) iochan_getdata(h);
+    IOCHAN parent_chan = line->user;
     association *newas;
     int res;
     HANDLE newHandle;
@@ -314,7 +746,7 @@ static void listener(IOCHAN h, int event)
 	yaz_log(YLOG_DEBUG, "Accept ok");
 
 	if (!(new_chan = iochan_create(cs_fileno(new_line), ir_session,
-				       EVENT_INPUT)))
+				       EVENT_INPUT, parent_chan->chan_id)))
 	{
 	    yaz_log(YLOG_FATAL, "Failed to create iochan");
             iochan_destroy(h);
@@ -322,7 +754,8 @@ static void listener(IOCHAN h, int event)
 	}
 
 	yaz_log(YLOG_DEBUG, "Creating association");
-	if (!(newas = create_association(new_chan, new_line)))
+	if (!(newas = create_association(new_chan, new_line,
+					 control_block.apdufile)))
 	{
 	    yaz_log(YLOG_FATAL, "Failed to create new assoc.");
             iochan_destroy(h);
@@ -384,12 +817,12 @@ void statserv_closedown()
 {
     IOCHAN p;
 
-    if (control_block.bend_stop)
-        (*control_block.bend_stop)(&control_block);
+    xml_config_bend_stop();
     for (p = pListener; p; p = p->next)
     {
         iochan_destroy(p);
     }
+    xml_config_close();
 }
 
 void sigterm(int sig)
@@ -427,6 +860,9 @@ static void listener(IOCHAN h, int event)
 	    iochan_setflags(h, EVENT_INPUT | EVENT_EXCEPT); /* reset listener */
 	    return;
 	}
+
+	yaz_log(log_session, "Connect from %s", cs_addrstr(new_line));
+
 	no_sessions++;
 	if (control_block.dynamic)
 	{
@@ -459,6 +895,7 @@ static void listener(IOCHAN h, int event)
 		return;
 	    }
 	}
+
 	if (control_block.threads)
 	{
 #if YAZ_POSIX_THREADS
@@ -502,12 +939,13 @@ static void *new_session (void *vp)
     association *newas;
     IOCHAN new_chan;
     COMSTACK new_line = (COMSTACK) vp;
+    IOCHAN parent_chan = new_line->user;
 
     unsigned cs_get_mask, cs_accept_mask, mask =  
 	((new_line->io_pending & CS_WANT_WRITE) ? EVENT_OUTPUT : 0) |
 	((new_line->io_pending & CS_WANT_READ) ? EVENT_INPUT : 0);
 
-    if (mask)    
+    if (mask)
     {
 	cs_accept_mask = mask;  /* accept didn't complete */
 	cs_get_mask = 0;
@@ -518,12 +956,14 @@ static void *new_session (void *vp)
 	cs_get_mask = mask = EVENT_INPUT;
     }
 
-    if (!(new_chan = iochan_create(cs_fileno(new_line), ir_session, mask)))
+    if (!(new_chan = iochan_create(cs_fileno(new_line), ir_session, mask,
+				   parent_chan->chan_id)))
     {
 	yaz_log(YLOG_FATAL, "Failed to create iochan");
 	return 0;
     }
-    if (!(newas = create_association(new_chan, new_line)))
+    if (!(newas = create_association(new_chan, new_line,
+				     control_block.apdufile)))
     {
 	yaz_log(YLOG_FATAL, "Failed to create new assoc.");
 	return 0;
@@ -566,9 +1006,11 @@ static void inetd_connection(int what)
 
     if ((line = cs_createbysocket(0, tcpip_type, 0, what)))
     {
-        if ((chan = iochan_create(cs_fileno(line), ir_session, EVENT_INPUT)))
+        if ((chan = iochan_create(cs_fileno(line), ir_session, EVENT_INPUT,
+				  0)))
         {
-            if ((assoc = create_association(chan, line)))
+            if ((assoc = create_association(chan, line,
+					    control_block.apdufile)))
             {
                 iochan_setdata(chan, assoc);
                 iochan_settimeout(chan, 60);
@@ -598,7 +1040,7 @@ static void inetd_connection(int what)
 /*
  * Set up a listening endpoint, and give it to the event-handler.
  */
-static int add_listener(char *where, int what)
+static int add_listener(char *where, int listen_id)
 {
     COMSTACK l;
     void *ap;
@@ -612,8 +1054,8 @@ static int add_listener(char *where, int what)
     else
 	mode = "static";
 
-    yaz_log(log_server, "Adding %s %s listener on %s", mode,
-	    what == PROTO_SR ? "SR" : "Z3950", where);
+    yaz_log(log_server, "Adding %s listener on %s id=%d", mode, where,
+	    listen_id);
 
     l = cs_create_host(where, 2, &ap);
     if (!l)
@@ -631,15 +1073,16 @@ static int add_listener(char *where, int what)
 	return -1;
     }
     if (!(lst = iochan_create(cs_fileno(l), listener, EVENT_INPUT |
-   	 EVENT_EXCEPT)))
+   	 EVENT_EXCEPT, listen_id)))
     {
     	yaz_log(YLOG_FATAL|YLOG_ERRNO, "Failed to create IOCHAN-type");
 	cs_close (l);
 	return -1;
     }
-    iochan_setdata(lst, l);
+    iochan_setdata(lst, l); /* user-defined data for listener is COMSTACK */
+    l->user = lst;  /* user-defined data for COMSTACK is listener chan */
 
-    /* Ensure our listener chain is setup properly */
+    /* Add listener to chain */
     lst->next = pListener;
     pListener = lst;
     return 0; /* OK */
@@ -657,15 +1100,35 @@ static void catchchld(int num)
 
 statserv_options_block *statserv_getcontrol(void)
 {
-    static statserv_options_block cb;
-
-    memcpy(&cb, &control_block, sizeof(cb));
-    return &cb;
+#ifdef WIN32
+    if (init_control_tls)
+	return (statserv_options_block *) TlsGetValue(current_control_tls);
+    else
+	return &control_block;
+#elif YAZ_POSIX_THREADS
+    if (init_control_tls)
+	return pthread_getspecific(current_control_tls);
+    else
+	return &control_block;
+#else
+    if (current_control_block)
+        return current_control_block;
+    return &control_block;
+#endif
 }
 
 void statserv_setcontrol(statserv_options_block *block)
 {
-    memcpy(&control_block, block, sizeof(*block));
+    chdir(gfs_root_dir);
+#ifdef WIN32
+    if (init_control_tls)
+	TlsSetValue(current_control_tls, block);
+#elif YAZ_POSIX_THREADS
+    if (init_control_tls)
+	pthread_setspecific(current_control_tls, block);
+#else
+    current_control_block = block;
+#endif
 }
 
 static void statserv_reset(void)
@@ -674,38 +1137,38 @@ static void statserv_reset(void)
 
 int statserv_start(int argc, char **argv)
 {
-    int ret = 0;
     char sep;
 #ifdef WIN32
     /* We need to initialize the thread list */
     ThreadList_Initialize();
 /* WIN32 */
 #endif
-    
+
+
 #ifdef WIN32
-    sep='\\';
+    sep = '\\';
 #else
-    sep='/';
+    sep = '/';
 #endif
     if ((me = strrchr (argv[0], sep)))
 	me++; /* get the basename */
     else
 	me = argv[0];
-    programname=argv[0];
+    programname = argv[0];
 
     if (control_block.options_func(argc, argv))
-        return(1);
+        return 1;
+
+    xml_config_open();
     
-    if (control_block.bend_start)
-        (*control_block.bend_start)(&control_block);
+    xml_config_bend_start();
+
 #ifdef WIN32
+    xml_config_add_listeners();
+
     yaz_log (log_server, "Starting server %s", me);
     if (!pListener && *control_block.default_listen)
-	add_listener(control_block.default_listen,
-		     control_block.default_proto);
-    
-    if (!pListener)
-	return 1;
+	add_listener(control_block.default_listen, 0);
 #else
 /* UNIX */
     if (control_block.inetd)
@@ -756,9 +1219,10 @@ int statserv_start(int argc, char **argv)
 	    open("/dev/null", O_RDWR);
 	    dup(0); dup(0);
 	}
+	xml_config_add_listeners();
+
 	if (!pListener && *control_block.default_listen)
-	    add_listener(control_block.default_listen,
-			 control_block.default_proto);
+	    add_listener(control_block.default_listen, 0);
 	
 	if (!pListener)
 	    return 1;
@@ -775,13 +1239,13 @@ int statserv_start(int argc, char **argv)
 	    fprintf(f, "%ld", (long) getpid());
 	    fclose(f);
 	}
-
+	
 	if (control_block.background)
 	    close(hand[1]);
 
-	yaz_log (log_server, "Starting server %s pid=%ld", programname,
-			(long) getpid());
-        
+
+	yaz_log (log_server, "Starting server %s pid=%ld", programname, 
+		 (long) getpid());
 #if 0
 	sigset_t sigs_to_block;
 	
@@ -812,18 +1276,16 @@ int statserv_start(int argc, char **argv)
     }
 /* UNIX */
 #endif
-    if ((pListener == NULL) && *control_block.default_listen)
-	add_listener(control_block.default_listen,
-		     control_block.default_proto);
-	
     if (pListener == NULL)
-	ret = 1;
-    else
-    {
-	yaz_log(YLOG_DEBUG, "Entering event loop.");
-        ret = event_loop(&pListener);
-    }
-    return ret;
+	return 1;
+    yaz_log(YLOG_DEBUG, "Entering event loop.");
+    return event_loop(&pListener);
+}
+
+static void option_copy(char *dst, const char *src)
+{
+    strncpy(dst, src ? src : "", 127);
+    dst[127] = '\0';
 }
 
 int check_options(int argc, char **argv)
@@ -835,13 +1297,14 @@ int check_options(int argc, char **argv)
     control_block.loglevel = yaz_log_mask_str(STAT_DEFAULT_LOG_LEVEL);
     yaz_log_init_level(control_block.loglevel);
 
-    while ((ret = options("1a:iszSTl:v:u:c:w:t:k:d:A:p:DC:",
+    get_logbits(1); 
+    while ((ret = options("1a:iszSTl:v:u:c:w:t:k:d:A:p:DC:f:",
 			  argv, argc, &arg)) != -2)
     {
     	switch (ret)
     	{
 	case 0:
-	    if (add_listener(arg, control_block.default_proto))
+	    if (add_listener(arg, 0))
                 return 1;  /* failed to create listener */
 	    break;
 	case '1':	 
@@ -871,27 +1334,29 @@ int check_options(int argc, char **argv)
 #endif
 	    break;
 	case 'l':
-	    strcpy(control_block.logfile, arg ? arg : "");
+	    option_copy(control_block.logfile, arg);
 	    yaz_log_init(control_block.loglevel, me, control_block.logfile);
 	    break;
 	case 'v':
-	    control_block.loglevel = yaz_log_mask_str_x(arg,control_block.loglevel);
+	    control_block.loglevel =
+		yaz_log_mask_str_x(arg,control_block.loglevel);
 	    yaz_log_init(control_block.loglevel, me, control_block.logfile);
+	    get_logbits(1); 
 	    break;
 	case 'a':
-	    strcpy(control_block.apdufile, arg ? arg : "");
+	    option_copy(control_block.apdufile, arg);
 	    break;
 	case 'u':
-	    strcpy(control_block.setuid, arg ? arg : "");
+	    option_copy(control_block.setuid, arg);
 	    break;
 	case 'c':
-	    strcpy(control_block.configname, arg ? arg : "");
+	    option_copy(control_block.configname, arg);
 	    break;
 	case 'C':
-	    strcpy(control_block.cert_fname, arg ? arg : "");
+	    option_copy(control_block.cert_fname, arg);
 	    break;
 	case 'd':
-	    strcpy(control_block.daemon_name, arg ? arg : "");
+	    option_copy(control_block.daemon_name, arg);
 	    break;
 	case 't':
 	    if (!arg || !(r = atoi(arg)))
@@ -923,12 +1388,15 @@ int check_options(int argc, char **argv)
             max_sessions = atoi(arg);
             break;
 	case 'p':
-	    if (strlen(arg) >= sizeof(control_block.pid_fname))
-	    {
-		yaz_log(YLOG_FATAL, "pid fname too long");
-		exit(1);
-	    }
-	    strcpy(control_block.pid_fname, arg);
+	    option_copy(control_block.pid_fname, arg);
+	    break;
+	case 'f':
+#if HAVE_XML2
+	    option_copy(control_block.xml_config, arg);
+#else
+	    fprintf(stderr, "%s: Option -f unsupported since YAZ is compiled without Libxml2 support\n", me);
+	    exit(1);
+#endif
 	    break;
 	case 'D':
 	    control_block.background = 1;
@@ -941,7 +1409,6 @@ int check_options(int argc, char **argv)
 	    return 1;
         }
     }
-    get_logbits(1); 
     return 0;
 }
 
@@ -964,13 +1431,10 @@ int statserv_main(int argc, char **argv,
 		  bend_initresult *(*bend_init)(bend_initrequest *r),
 		  void (*bend_close)(void *handle))
 {
-    statserv_options_block *cb = statserv_getcontrol();
-    
+    struct statserv_options_block *cb = &control_block;
     cb->bend_init = bend_init;
     cb->bend_close = bend_close;
 
-    statserv_setcontrol(cb);
-
     /* Lets setup the Arg structure */
     ArgDetails.argc = argc;
     ArgDetails.argv = argv;
@@ -1011,12 +1475,10 @@ int statserv_main(int argc, char **argv,
 		  void (*bend_close)(void *handle))
 {
     int ret;
-    statserv_options_block *cb = statserv_getcontrol();
-    
-    cb->bend_init = bend_init;
-    cb->bend_close = bend_close;
 
-    statserv_setcontrol(cb);
+    control_block.bend_init = bend_init;
+    control_block.bend_close = bend_close;
+
     ret = statserv_start (argc, argv);
     statserv_closedown ();
     statserv_reset();
