@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: tcpdchk.c,v 1.4 2004/12/13 14:21:56 heikki Exp $
+ * $Id: tcpdchk.c,v 1.7 2005/01/17 10:29:25 adam Exp $
  */
 /**
  * \file tcpdchk.c
@@ -12,6 +12,23 @@
 #include <stdio.h>
 #include <string.h>
 
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef WIN32
+#include <winsock.h>
+#else
+#include <netinet/in.h>
+#include <netdb.h>
+#include <arpa/inet.h>
+#include <netinet/tcp.h>
+#endif
+
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #include <yaz/yconfig.h>
 #include <yaz/comstack.h>
 #include <yaz/statserv.h>
@@ -22,7 +39,6 @@
 #include <syslog.h>
 #include <tcpd.h>
 
-
 int allow_severity = LOG_INFO;  /* not YLOG !! */
 int deny_severity = LOG_WARNING;
 
